@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2004 Sistina Software, Inc. All rights reserved.  
+ * Copyright (C) 2001-2004 Sistina Software, Inc. All rights reserved.
  * Copyright (C) 2004-2007 Red Hat, Inc. All rights reserved.
  *
  * This file is part of LVM2.
@@ -648,6 +648,11 @@ static int _process_all_devs(struct cmd_context *cmd, void *handle,
 	int ret_max = 0;
 	int ret = 0;
 
+	if (!scan_vgs_for_pvs(cmd)) {
+		stack;
+		return ECMD_FAILED;
+	}
+
 	if (!(iter = dev_iter_create(cmd->filter, 1))) {
 		log_error("dev_iter creation failed");
 		return ECMD_FAILED;
@@ -736,13 +741,13 @@ int process_each_pv(struct cmd_context *cmd, int argc, char **argv,
 					continue;
 				}
 
-        			/*
-			         * If a PV has no MDAs it may appear to be an
+				/*
+				 * If a PV has no MDAs it may appear to be an
 				 * orphan until the metadata is read off
 				 * another PV in the same VG.  Detecting this
 				 * means checking every VG by scanning every
 				 * PV on the system.
-			         */
+				 */
 				if (!scanned && is_orphan(pv)) {
 					if (!scan_vgs_for_pvs(cmd)) {
 						stack;
@@ -978,10 +983,8 @@ static int _parse_pes(struct dm_pool *mem, char *c, struct list *pe_ranges,
 
 	/* Default to whole PV */
 	if (!c) {
-		if (!_add_pe_range(mem, pvname, pe_ranges, UINT32_C(0), size)) {
-			stack;
-			return 0;
-		}
+		if (!_add_pe_range(mem, pvname, pe_ranges, UINT32_C(0), size))
+			return_0;
 		return 1;
 	}
 
@@ -1026,10 +1029,8 @@ static int _parse_pes(struct dm_pool *mem, char *c, struct list *pe_ranges,
 			return 0;
 		}
 
-		if (!_add_pe_range(mem, pvname, pe_ranges, start, end - start + 1)) {
-			stack;
-			return 0;
-		}
+		if (!_add_pe_range(mem, pvname, pe_ranges, start, end - start + 1))
+			return_0;
 
 	}
 
@@ -1084,10 +1085,8 @@ static int _create_pv_entry(struct dm_pool *mem, struct pv_list *pvl,
 
 	/* Determine selected physical extents */
 	if (!_parse_pes(mem, colon, new_pvl->pe_ranges, pv_dev_name(pvl->pv),
-			pvl->pv->pe_count)) {
-		stack;
-		return 0;
-	}
+			pvl->pv->pe_count))
+		return_0;
 
 	return 1;
 }
@@ -1124,10 +1123,8 @@ struct list *create_pv_list(struct dm_pool *mem, struct volume_group *vg, int ar
 							tagname)) {
 					if (!_create_pv_entry(mem, pvl, NULL,
 							      allocatable_only,
-							      r)) {
-						stack;
-						return NULL;
-					}
+							      r))
+						return_NULL;
 				}
 			}
 			continue;
@@ -1149,10 +1146,8 @@ struct list *create_pv_list(struct dm_pool *mem, struct volume_group *vg, int ar
 				"Volume Group \"%s\"", pvname, vg->name);
 			return NULL;
 		}
-		if (!_create_pv_entry(mem, pvl, colon, allocatable_only, r)) {
-			stack;
-			return NULL;
-		}
+		if (!_create_pv_entry(mem, pvl, colon, allocatable_only, r))
+			return_NULL;
 	}
 
 	if (list_empty(r))
@@ -1239,6 +1234,17 @@ int apply_lvname_restrictions(const char *name)
 	return 1;
 }
 
+int is_reserved_lvname(const char *name)
+{
+	int rc, old_suppress;
+
+	old_suppress = log_suppress(2);
+	rc = !apply_lvname_restrictions(name);
+	log_suppress(old_suppress);
+
+	return rc;
+}
+
 /*
  * Set members of struct vgcreate_params from cmdline.
  * Do preliminary validation with arg_*() interface.
@@ -1261,7 +1267,7 @@ int fill_vg_create_params(struct cmd_context *cmd,
 	    arg_uint_value(cmd, physicalextentsize_ARG, vp_def->extent_size);
 
 	if (arg_count(cmd, clustered_ARG))
-        	vp_new->clustered =
+		vp_new->clustered =
 			!strcmp(arg_str_value(cmd, clustered_ARG,
 					      vp_def->clustered ? "y":"n"), "y");
 	else
