@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2004 Sistina Software, Inc. All rights reserved. 
+ * Copyright (C) 2001-2004 Sistina Software, Inc. All rights reserved.
  * Copyright (C) 2004-2007 Red Hat, Inc. All rights reserved.
  *
  * This file is part of LVM2.
@@ -25,10 +25,8 @@ static int _remove_pv(struct volume_group *vg, struct pv_list *pvl)
 		return 0;
 	}
 
-	if (!id_write_format(&pvl->pv->id, uuid, sizeof(uuid))) {
-		stack;
-		return 0;
-	}
+	if (!id_write_format(&pvl->pv->id, uuid, sizeof(uuid)))
+		return_0;
 
 	log_verbose("Removing PV with UUID %s from VG %s", uuid, vg->name);
 
@@ -91,22 +89,18 @@ static int _remove_lv(struct cmd_context *cmd, struct logical_volume *lv,
 		}
 
 		*list_unsafe = 1;	/* May remove caller's lvht! */
-		if (!vg_remove_snapshot(cow)) {
-			stack;
-			return 0;
-		}
+		if (!vg_remove_snapshot(cow))
+			return_0;
 		log_verbose("Removing LV %s from VG %s", cow->name,
 			    lv->vg->name);
-		if (!lv_remove(cow)) {
-			stack;
-			return 0;
-		}
+		if (!lv_remove(cow))
+			return_0;
 
 		first = 0;
 	}
 
 	/*
-	 * If LV is active, replace it with error segment 
+	 * If LV is active, replace it with error segment
 	 * and add to list of LVs to be removed later.
 	 * Doesn't apply to snapshots/origins yet - they're already deactivated.
 	 */
@@ -129,10 +123,8 @@ static int _remove_lv(struct cmd_context *cmd, struct logical_volume *lv,
 	} else {
 		/* Remove LV immediately. */
 		log_verbose("Removing LV %s from VG %s", lv->name, lv->vg->name);
-		if (!lv_remove(lv)) {
-			stack;
-			return 0;
-		}
+		if (!lv_remove(lv))
+			return_0;
 	}
 
 	return 1;
@@ -177,10 +169,8 @@ static int _make_vg_consistent(struct cmd_context *cmd, struct volume_group *vg)
 						only_mirror_images_found = 0;
 						continue;
 					}
-					if (!_remove_lv(cmd, lv, &list_unsafe, &lvs_changed)) {
-						stack;
-						return 0;
-					}
+					if (!_remove_lv(cmd, lv, &list_unsafe, &lvs_changed))
+						return_0;
 					if (list_unsafe)
 						goto restart_loop;
 				}
@@ -198,10 +188,8 @@ static int _make_vg_consistent(struct cmd_context *cmd, struct volume_group *vg)
 		pvl = list_item(pvh, struct pv_list);
 		if (pvl->pv->dev)
 			continue;
-		if (!_remove_pv(vg, pvl)) {
-			stack;
-			return 0;
-		}
+		if (!_remove_pv(vg, pvl))
+			return_0;
 	}
 
 	/* VG is now consistent */
@@ -290,10 +278,8 @@ static int _make_vg_consistent(struct cmd_context *cmd, struct volume_group *vg)
 
 			if ((mimages != mirrored_seg->area_count) || remove_log){
 				if (!reconfigure_mirror_images(mirrored_seg, mimages,
-							       NULL, remove_log)) {
-					stack;
-					return 0;
-				}
+							       NULL, remove_log))
+					return_0;
 
 				if (!vg_write(vg)) {
 					log_error("Failed to write out updated "
@@ -353,10 +339,8 @@ static int _make_vg_consistent(struct cmd_context *cmd, struct volume_group *vg)
 				    lvl->lv->vg->name);
 				/* Skip LVs already removed by mirror code */
 				if (find_lv_in_vg(vg, lvl->lv->name) &&
-				    !lv_remove(lvl->lv)) {
-				stack;
-				return 0;
-			}
+				    !lv_remove(lvl->lv))
+					return_0;
 		}
 	}
 
@@ -369,6 +353,8 @@ static int _vgreduce_single(struct cmd_context *cmd, struct volume_group *vg,
 			    void *handle __attribute((unused)))
 {
 	struct pv_list *pvl;
+	struct volume_group *orphan_vg;
+	int consistent = 1;
 	const char *name = pv_dev_name(pv);
 
 	if (pv_pe_alloc_count(pv)) {
@@ -382,21 +368,29 @@ static int _vgreduce_single(struct cmd_context *cmd, struct volume_group *vg,
 		return ECMD_FAILED;
 	}
 
+	if (!lock_vol(cmd, VG_ORPHANS, LCK_VG_WRITE | LCK_NONBLOCK)) {
+		log_error("Can't get lock for orphan PVs");
+		return ECMD_FAILED;
+	}
+
 	pvl = find_pv_in_vg(vg, name);
 
-	if (!archive(vg))
+	if (!archive(vg)) {
+		unlock_vg(cmd, VG_ORPHANS);
 		return ECMD_FAILED;
+	}
 
 	log_verbose("Removing \"%s\" from volume group \"%s\"", name, vg->name);
 
 	if (pvl)
 		list_del(&pvl->list);
 
-	pv->vg_name = ORPHAN;
+	pv->vg_name = vg->fid->fmt->orphan_vg_name;
 	pv->status = ALLOCATABLE_PV;
 
 	if (!dev_get_size(pv_dev(pv), &pv->size)) {
 		log_error("%s: Couldn't get size.", pv_dev_name(pv));
+		unlock_vg(cmd, VG_ORPHANS);
 		return ECMD_FAILED;
 	}
 
@@ -404,9 +398,24 @@ static int _vgreduce_single(struct cmd_context *cmd, struct volume_group *vg,
 	vg->free_count -= pv_pe_count(pv) - pv_pe_alloc_count(pv);
 	vg->extent_count -= pv_pe_count(pv);
 
+	if(!(orphan_vg = vg_read(cmd, vg->fid->fmt->orphan_vg_name, NULL, &consistent)) ||
+	   !consistent) {
+		log_error("Unable to read existing orphan PVs");
+		unlock_vg(cmd, VG_ORPHANS);
+		return ECMD_FAILED;
+	}
+
+	if (!vg_split_mdas(cmd, vg, orphan_vg) || !vg->pv_count) {
+		log_error("Cannot remove final metadata area on \"%s\" from \"%s\"",
+			  name, vg->name);
+		unlock_vg(cmd, VG_ORPHANS);
+		return ECMD_FAILED;
+	}
+
 	if (!vg_write(vg) || !vg_commit(vg)) {
 		log_error("Removal of physical volume \"%s\" from "
 			  "\"%s\" failed", name, vg->name);
+		unlock_vg(cmd, VG_ORPHANS);
 		return ECMD_FAILED;
 	}
 
@@ -414,9 +423,11 @@ static int _vgreduce_single(struct cmd_context *cmd, struct volume_group *vg,
 		log_error("Failed to clear metadata from physical "
 			  "volume \"%s\" "
 			  "after removal from \"%s\"", name, vg->name);
+		unlock_vg(cmd, VG_ORPHANS);
 		return ECMD_FAILED;
 	}
 
+	unlock_vg(cmd, VG_ORPHANS);
 	backup(vg);
 
 	log_print("Removed \"%s\" from volume group \"%s\"", name, vg->name);
